package com.api.games.service.impl;

import com.api.common.JSONUtils;
import com.api.common.UtilFun;
import com.api.core.response.Result;
import com.api.core.response.ResultGenerator;
import com.api.core.service.AbstractService;
import com.api.games.dao.GameAnswerMapper;
import com.api.games.dao.GameLogMapper;
import com.api.games.dao.GamePlayTimeMapper;
import com.api.games.dao.UserConfigMapper;
import com.api.games.model.GameAnswer;
import com.api.games.model.GameLog;
import com.api.games.model.UserConfig;
import com.api.games.service.GameAnswerService;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.Calendar;
import java.util.Date;
import java.util.List;


/**
 * Created by wanghuiwen on 2020/02/23.
 */
@Service
@Transactional
public class GameAnswerServiceImpl extends AbstractService<GameAnswer> implements GameAnswerService {
    @Resource
    private GameAnswerMapper gameAnswerMapper;
    @Resource
    private GameLogMapper gameLogMapper;
    @Resource
    private UserConfigMapper userConfigMapper;
    @Resource
    GamePlayTimeMapper gamePlayTimeMapper;

    @Override
    public Result add(GameLog gameLog, String gameAnswer) {
        List<GameAnswer> res = JSONUtils.json2list(gameAnswer, GameAnswer.class);
        UserConfig config = userConfigMapper.findByUid(gameLog.getUserId());
        int count = config.getCorrect() + config.getSupporting();
        //完成度
        gameLog.setSchedule(BigDecimal.valueOf(res.size() / count));

        int days = getDays(config);
        List<GameLog> logCount = gameLogMapper.listUidDate(gameLog.getUserId(),UtilFun.DateToString(new Date(), UtilFun.YMD));
        gameLog.setSign(days + "-" + (logCount.size() + 1));
        gameLogMapper.insertSelective(gameLog);
        for (GameAnswer a : res) {
            a.setGameLogId(gameLog.getId());
        }
        gameAnswerMapper.insertList(res);
        return ResultGenerator.genSuccessResult();
    }

    @Override
    public Result add(Long gamePlayId, GameLog gameLog, String gameAnswer) {
        if (gamePlayTimeMapper.deleteByPrimaryKey(gamePlayId) != 0)
            return add(gameLog, gameAnswer);
        else
            return new Result("無此條遊戲時間記錄");
    }

    private int getDays(UserConfig config){
        try {
            Calendar today = Calendar.getInstance();
            Calendar scaleStart = Calendar.getInstance();
            Date scale = UtilFun.StringToDate(config.getScaleStart(), UtilFun.YYYYMMDDHHMMSS);
            if (scale != null) {
                scaleStart.setTime(scale);
                if (today.before(scaleStart) || isSameDay(today, scaleStart))
                    return 1;

                int result = 0;
                while (!isSameDay(today, scaleStart)){
                    result ++;
                    today.add(Calendar.DATE, -1);
                }
                return result;
            }
        } catch (Exception e){
            e.printStackTrace();
        }

        return 1;
    }

    private boolean isSameDay(Calendar left, Calendar right){
        return left.get(Calendar.YEAR) == right.get(Calendar.YEAR)
            && left.get(Calendar.MONTH) == right.get(Calendar.MONTH)
            && left.get(Calendar.DAY_OF_MONTH) == right.get(Calendar.DAY_OF_MONTH);
    }
}
